import { useState } from 'react';

import useIsDesktopFallback from '../../../utils/hooks/useIsDesktopFallback';

/**
 * Hook to create standard tooltip object that can be provided to both
 * `TooltipDisplay` and `TooltipTrigger` to coordinate tooltip activation and
 * content.
 *
 * You can pass in a configuration object to use to configure various aspects
 * of the tooltip. For example, if you want to reuse one display across many
 * triggers and you want to use the same component to render each tooltip's
 * content, you might provide a default value for `Component` and then have
 * each trigger set its own `props` to pass to that component when the trigger
 * is active.
 *
 * The configuration object can also specify an `isDesktop` flag, which you
 * should try to provide if your application is already computing this value,
 * say, in a Recoil store. If `isDesktop` is _not_ provided, this hook will set
 * up a resize listener to keep track of whether we're at a mobile or desktop
 * width. The listener with use the `breakpoint` configuration option to make
 * that classification.
 */
const useTooltip = function useTooltip(opts = {}) {
  const {
    isDesktop: providedIsDesktop,
    breakpoint,
    ...restOpts
  } = opts;

  const isDesktop = useIsDesktopFallback({ providedIsDesktop, breakpoint });

  const [config, setConfig] = useState({ active: false });

  // An activation function that will trigger this tooltip with optional prop
  // overrides to customize the tooltip for the trigger.
  const activate = (activateProps = {}) => {
    const {
      content = opts.content,
      props = opts.props,
      ...restProps
    } = activateProps;

    setConfig({
      ...restProps,
      content,
      props,
      active: true,
    });
  };

  // A deactivation function that will turn the tooltip off
  const deactivate = () => {
    setConfig({ active: false });
  };

  return {
    config: {
      ...restOpts,
      ...config,
      isDesktop,
    },
    trigger: {
      activate,
      deactivate,
    },
  };
};

export default useTooltip;
